
#include <stdio.h>        /*  printf()                         */
#include <errno.h>        /*  errno                            */
#include <string.h>       /*  strerror()                       */
#include <unistd.h>       /*  getuid() write() close()         */
#include <fcntl.h>        /*  open() flags                     */
#include <sys/stat.h>     /*  lstat()                          */
#include <sys/ioctl.h>    /*  ioctl()                          */
#include <linux/hdreg.h>  /*  ioctl() flag: HDIO_GET_IDENTITY  */

#define NAME "identification"

int usage(void)
{
	printf("%s - dump identification block from block device to output file \"indentity.bin\"\n", NAME);
	printf("  Note : root access required\n");
	printf("  Usage:\n\t%s /path/to/block/device\n", NAME);
	return -1;
}

int main(int argc, const char *argv[])
{
	if ((argc != 2) || (getuid() != 0)) {
		return usage();
	}
	
	char *file = argv[1];
	struct stat st;
	
	/* verify block device file */
	if ((lstat(file, &st) != 0) || (!(S_ISBLK(st.st_mode))) || (major(st.st_rdev) != 8)) {
		fprintf(stderr, "%s: %s: cannot get information about block device file %s\n", NAME, "ERROR", file);
		return -1;
	}
	
	int blkd;
	
	/* open device for ioctl() */
	if ((blkd = open(file, O_RDONLY)) == -1) {
		fprintf(stderr, "%s: %s: cannot open block device file \"%s\" (%s)\n", NAME, "ERROR", file, strerror(errno));
		return -1;
	}
	
	int block = 512;
	unsigned char identity[block];
	
	/* get information via ioctl() */
	if (ioctl(blkd, HDIO_GET_IDENTITY, identity) == -1) {
		fprintf(stderr, "%s: %s: cannot get identification information about block device \"%s\" (%s)\n", NAME, "ERROR", file, strerror(errno));
		return -1;
	}
	
	char *ofile = "identity.bin";
	int output = 0;
	
	/* open file for saving information */
	if ((output = open(ofile, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		fprintf(stderr, "%s: %s: cannot open file \"%s\" for saving information about block device (%s)\n", NAME, "ERROR", ofile, strerror(errno));
		return -1;
	}
	
	int w_status = 0;
	
	/* write information in file */
	w_status = write(output, identity, block);
	if ((w_status == -1) || (w_status != block)) {
		fprintf(stderr, "%s: %s: cannot save information about block device in file \"%s\" (%s)\n", NAME, "ERROR", ofile, strerror(errno));
		return -1;
	}
	
	/* close files */
	if (close(output) == -1) {
		fprintf(stderr, "%s: %s: cannot close file \"%s\" with information (%s)\n", NAME, "ERROR", ofile, strerror(errno));
	}
	
	if (close(blkd) == -1) {
		fprintf(stderr, "%s: %s: cannot close block device file \"%s\" (%s)\n", NAME, "ERROR", file, strerror(errno));
	}
	
	return 0;
}

