
#include <stdio.h>        /*  printf()                         */
#include <errno.h>        /*  errno                            */
#include <ctype.h>        /*  isspace()                        */
#include <string.h>       /*  strerror()                       */
#include <unistd.h>       /*  getuid() write() close()         */
#include <fcntl.h>        /*  open() flags                     */
#include <sys/stat.h>     /*  lstat()                          */
#include <sys/ioctl.h>    /*  ioctl()                          */
#include <linux/hdreg.h>  /*  ioctl() flag: HDIO_GET_IDENTITY  */

#define NAME "hddriveid"

int usage(void)
{
	printf("%s - basic information about hard drive\n", NAME);
	printf("  Note : root access required\n");
	printf("  Usage:\n\t%s /path/to/block/device\n", NAME);
	return -1;
}

char *strip_name(char *str)
{
	int i;
	int end = 0;
	
	for (i = 0; str[i] != 0; i++) {
		if (!isspace (str[i]) || (isspace (str[i]) && !isspace (str[i+1]) && str[i+1])) {
			str[end] = str[i];
			end++;
		}
	}
	
	str[end] = 0;
	return strdup(str);
}

int main(int argc, const char *argv[])
{
	/* check for args and root access */
	if ((argc != 2) || (getuid() != 0)) {
		return usage();
	}
	
	char *file = argv[1];
	
	/* verify block device file */
	struct stat st;
	if ((lstat(file, &st) != 0) || (!(S_ISBLK(st.st_mode))) || (major(st.st_rdev) != 8)) {
		fprintf(stderr, "%s: %s: cannot get information about block device file %s\n", NAME, "ERROR", file);
		return -1;
	}
	
	int blkd;
	
	/* open device for ioctl() */
	if ((blkd = open(file, O_RDONLY)) == -1) {
		fprintf(stderr, "%s: %s: cannot open block device file %s (%s)\n", NAME, "ERROR", file, strerror(errno));
		return -1;
	}
	
	struct hd_driveid hd;
	
	/* get information via ioctl() */
	if (ioctl(blkd, HDIO_GET_IDENTITY, &hd) == -1) {
		fprintf(stderr, "%s: %s: cannot get identification information about block device (%s)\n", NAME, "ERROR", strerror(errno));
		return -1;
	}
	
	char model[40];
	strncpy(model, hd.model, 40);
	model[40] = '\0';
	int i; int end = 0;
	
	/* cleaning model name */
	for (i = 0; i < 40 && model[i] != 0; i++) {
		if (!isspace(model[i]) || (isspace(model[i]) && !isspace(model[i+1]) && model[i+1])) {
			model[end] = model[i];
			end++;
		}
	}
	model[end] = 0;
	strdup(model);
	
	/* cleaning serial name */
	char serial[20];
	strncpy(serial, hd.serial_no, 20);
	serial[20] = '\0';
	
	/* displaying collected information */
	printf("\tDevice file   : %s\n", file);
	printf("\tModel  number : %s\n", model);
	printf("\tSerial number :%s\n", strip_name(serial));
	
	return 0;
}

