
#define _LARGEFILE64_SOURCE

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <linux/hdreg.h>
#include <linux/types.h>
#include <errno.h>

#include <arpa/inet.h>

int endian_swap(unsigned short x)
{
    return (x>>8) | (x<<8);
}

typedef struct slots_s {
	__u8    aflag;
	__u8    chs_b [  3];
	__u8    type;
	__u8    chs_e [  3];
	__u32   lba;
	__u32   size;
} __attribute__ ((packed)) slots_t;

typedef struct partitions_s {
	__u8    boot  [446];
	slots_t slot  [  4];
	__u16   sign;
} __attribute__ ((packed)) partitions_t;

int ReadSubPart(int blkd, partitions_t *partitions, int i, unsigned long long Offset )
{
	int j = 0;

	if (lseek(blkd, ( (off_t)partitions->slot[i].lba + Offset )*512, SEEK_SET) == -1) {
		printf("lseek: %s\n", strerror(errno));
		return -1;
	}

	printf("lseek: %lld\n", abs + partitions->slot[i].lba );

	if (read(blkd, partitions, 512) != 512) {
		printf("read: %s\n", strerror(errno));
		return -1;
	}

	for (j = 0; j < 4; j++) {
		printf("\tslot %d : %.2hhX\t", j, partitions->slot[j].type);
		printf("lba  : %d[%lld]\t", partitions->slot[j].lba, partitions->slot[j].lba+Offset);
		printf("size : %d\n", partitions->slot[j].size);
	}
	for (j = 0; j < 4; j++) {
		if (partitions->slot[j].type == 0x05) {
			ReadSubPart( blkd, partitions, j, Offset);
		}
	}
}

int main(int argc, char const* argv[])
{
	partitions_t *partitions;
	partitions = malloc(512);
	memset(partitions, 0, 512);
	
	int blkd = open(argv[1], O_RDONLY | O_LARGEFILE);
	if (blkd == -1) {
		printf("open: %s\n", strerror(errno));
		return -1;
	}
	if (read(blkd, partitions, 512) != 512) {
		printf("read: %s\n", strerror(errno));
		return -1;
	}
	
	printf("\nMBR sign:\n");
	printf("\traw   : %hX\n", partitions->sign);
	printf("\tntohs : %hX\n", ntohs(partitions->sign));
	printf("\tswap  : %hX\n", endian_swap(ntohs(partitions->sign)));
	
	int i = 0, j = 0;
	printf("\nPartitions :\n");
	for (i = 0; i < 4; i++) {
		printf("\tslot %d : %.2hhX\t", i, partitions->slot[i].type);
		printf("lba  : %d\t", partitions->slot[i].lba);
		printf("size : %d\n", partitions->slot[i].size);
	}
	
	for (i = 0; i < 4; i++) {
		if (partitions->slot[i].type == 0x05) {
			ReadSubPart( blkd, partitions, i, partitions->slot[i].lba );
		}
	}
	
	close(blkd);
	free(partitions);
	
	return 0;
}

