
#define _LARGEFILE64_SOURCE

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <linux/hdreg.h>
#include <linux/types.h>
#include <errno.h>

#include <arpa/inet.h>

int endian_swap(unsigned short x)
{
    return (x>>8) | (x<<8);
}

typedef struct slots_s {
	__u8    aflag;
	__u8    chs_b [  3];
	__u8    type;
	__u8    chs_e [  3];
	__u32   lba;
	__u32   size;
} __attribute__ ((packed)) slots_t;

typedef struct partitions_s {
	__u8    boot  [446];
	slots_t slot  [  4];
	__u16   sign;
} __attribute__ ((packed)) partitions_t;

int main(int argc, char const* argv[])
{
	partitions_t *partitions;
	printf("sizeof(partitions_t) = %zd\n", sizeof(partitions_t));
	partitions = malloc(512);
	memset(partitions, 0, 512);
	//char *s;// = "string"
	//s = malloc(512);
	int blkd = open(argv[1], O_RDONLY | O_LARGEFILE);
	if (blkd == -1) {
		printf("open: %s\n", strerror(errno));
		return -1;
	}
	if (read(blkd, partitions, 512) != 512) {
		printf("read: %s\n", strerror(errno));
		return -1;
	}
	
	printf("\nMBR sign:\n");
	printf("\traw   : %hX\n", partitions->sign);
	printf("\tntohs : %hX\n", ntohs(partitions->sign));
	printf("\tswap  : %hX\n", endian_swap(ntohs(partitions->sign)));
	
	int i = 0, j = 0;
	printf("\nPartitions :\n");
	for (i = 0; i < 4; i++) {
		printf("\tslot %d : %.2hhX\n", i, partitions->slot[i].type);
		printf("\t\tlba  : %d\n", partitions->slot[i].lba);
		printf("\t\tsize : %d\n", partitions->slot[i].size);
	}

	partitions_t *partitions2;
	//printf("sizeof(partitions_t) = %zd\n", sizeof(partitions_t));
	partitions2 = malloc(512);
	memset(partitions2, 0, 512);
	off_t offset;
	for (i = 0; i < 4; i++) {
		if (partitions->slot[i].type == 0x05) {
			offset = (off_t) partitions->slot[i].lba * 512;
			printf("lba seek: %d\n", partitions->slot[i].lba);
			//offset = 646002507776;
			if (lseek(blkd, offset, SEEK_SET) == -1) {
				printf("lseek: %s\n", strerror(errno));
			}
			printf("lseek: %ld : %d\n", offset, offset/512);
			if (read(blkd, partitions2, 512) != 512) {
				printf("read: %s\n", strerror(errno));
			}
			for (j = 0; j < 4; j++) {
				printf("\tslot %d : %.2hhX\n", j, partitions2->slot[j].type);
				printf("\t\tlba  : %d\n", partitions2->slot[j].lba);
				printf("\t\tsize : %d\n", partitions2->slot[j].size);
			}
		}
	}
	
	close(blkd);
	free(partitions);
	
	return 0;
}

