
#include <stdio.h>           /*  printf()                         */
#include <string.h>          /*  memset()                         */
#include <stdlib.h>          /*  malloc()                         */
#include <unistd.h>          /*  getuid() write() close()         */
#include <fcntl.h>           /*  open() flags                     */
#include <ext2fs/ext2_fs.h>  /*  struct ext2_super_block          */

#define SBLOCK_OFFSET 1024
#define SBLOCK_SIZE   sizeof(struct ext2_super_block)

#define NAME "extinfo-simple"

int usage(void)
{
	printf("%s - show LABEL and UUID of EXT partition\n", NAME);
	printf("  Note : root access required\n");
	printf("  Usage:\n\t%s /path/to/block/device/with/extFS\n", NAME);
	return -1;
}

int main(int argc, char const* argv[])
{
	if ((argc != 2) || (getuid() != 0)) {
		return usage();
	}
	
	char *file = argv[1];
	struct ext2_super_block *sblock = (struct ext2_super_block *)malloc(SBLOCK_SIZE);
	memset(sblock, 0, sizeof(&sblock));
	
	/* open block device for reading */
	int blkd = open(file, O_RDONLY);
	
	/* seek to superblock location */
	lseek(blkd, SBLOCK_OFFSET, SEEK_CUR);
	
	/* read superblock content */
	read(blkd, sblock, SBLOCK_SIZE);
	
	/* close block file */
	close(blkd);
	
	/* show superblock information */
	printf("\tLABEL : %s\n", sblock->s_volume_name);
	printf("\tUUID  : ");
	/* routine making pretty output for UUID */
	int i = 0;
	for (i = 0; i < 16; i++) {
		if ((i == 4) || (i == 6) || (i == 8) || (i == 10)) {
			printf("-");
		}
		printf("%hx", sblock->s_uuid[i]);
	}
	printf("\n");
	
	return 0;
}

